@@ -27,23 +27,26 @@ def missing_relation_error(entity, rtype):
                                         'eid': entity.eid}}
     return ValidationError(entity.eid, errors)
 
-# initialization hooks #########################################################
-
 def _generate_vc_data(gen, linkedvar, rel):
     linkedvar.accept(gen)
     return 'VERSION_DATA(%s)' % linkedvar._q_sql
 
+# initialization hooks #########################################################
+
 class ServerStartupHook(Hook):
     """synchronize repository on server startup, and install attribute map
     on the system source sql generator
     """
     events = ('server_startup',)
     def call(self, repo):
-        bridge.import_content(repo)
+        try:
+            bridge.import_content(repo)
+        except:
+            self.exception('error while trying to import vcs content')
         repo.system_source.map_attribute('VersionContent', 'data',
                                          _generate_vc_data)
-        repo.looping_task(self.config.get('check-revision-interval', 5*60),
-                          lambda repo=repo: bridge.import_content(repo))
+        repo.looping_task(repo.config.get('check-revision-interval', 5*60),
+                          bridge.import_content, repo)
 
 
 class AddRepositoryHook(Hook):
@@ -100,11 +103,18 @@ class VCTransactionOp(Operation):
     def precommit_event(self):
         bridge.set_at_revision(self.session, self.revision.eid)
 
+    def revertprecommit_event(self):
+        transactions = self.session.transaction_data.setdefault('vctransactions', {})
+        for transaction in transactions.itervalues():
+            transaction.rollback()
+
     def commit_event(self):
         transactions = self.session.transaction_data.setdefault('vctransactions', {})
         for transaction in transactions.itervalues():
             transaction.commit()
 
+    revertcommit_event = revertprecommit_event
+
     def rollback_event(self):
         transactions = self.session.transaction_data.setdefault('vctransactions', {})
         for transaction in transactions.itervalues():
@@ -177,12 +187,12 @@ class AddVersionContentHook(Hook):
             vcsrepohdlr = bridge.repository_handler(vf.repository.eid)
             return Binary(vcsrepohdlr.file_content(
                 vf.directory, vf.name, vcsrepohdlr.imported_revision))
-        try:
-            # data not stored in the local database, pop it
-            return entity.pop('data')
-        except KeyError, ex:
+        # data not stored in the local database, pop it
+        data = entity.posted_data(fetch=False)
+        if not data:
             # missing data
             raise ValidationError(entity, {'data': session.__('required attribute')})
+        return data
 
 
 class AfterAddVersionContentHook(Hook):
@@ -192,6 +202,7 @@ class AfterAddVersionContentHook(Hook):
         # restore data for full text indexation
         entity['data'] = session.transaction_data.pop((entity.eid, 'data'))
 
+
 class AddDeletedVersionContentHook(Hook):
     events = ('before_add_entity',)
     accepts = ('DeletedVersionContent',)
@@ -218,6 +229,20 @@ class AddVersionedFileHook(Hook):
             return
         CheckVersionedFileOp(session, entity=entity)
 
+# ancestor_revision synchro ####################################################
+
+class AddParentRevisionHook(Hook):
+    events = ('after_add_relation',)
+    accepts = ('parent_revision',)
+
+    def call(self, session, fromeid, rtype, toeid):
+        session.repo.glob_add_relation(session,
+                                       fromeid, 'ancestor_revision', toeid)
+        session.unsafe_execute(
+            'SET R ancestor_revision AR '
+            'WHERE R eid %(rev)s, PR eid %(prev)s, PR ancestor_revision AR',
+            {'rev': fromeid, 'prev': toeid}, 'rev')
+
 
 # safety belts #################################################################
 
@@ -268,7 +293,6 @@ class CheckImmutalbeAttributeHook(Hook):
                 continue
             if '%s.%s' % (entity.id, attr) in IMMUTABLE_ATTRIBUTES:
                 raise QueryError('%s attribute is not editable' % attr)
-        self.local_update_entity(session, entity)
 
 
 class UpdateRepositoryHook(Hook):
@@ -277,11 +301,15 @@ class UpdateRepositoryHook(Hook):
     accepts = ('Repository',)
     def call(self, session, entity):
         # XXX check value actually changed
-        if 'path' in entity:
+        try:
+            edited = entity.edited_attributes
+        except AttributeError:
+            edited = entity
+        if 'path' in edited:
             msg = session._('updating path attribute of a repository isn\'t '
                             'supported. Delete it and add a new one.')
             raise QueryError(msg)
-        if 'type' in entity:
+        if 'type' in edited:
             msg = session._('updating type attribute of a repository isn\'t '
                             'supported. Delete it and add a new one.')
             raise QueryError(msg)
