@@ -6,6 +6,7 @@
 """
 __docformat__ = "restructuredtext en"
 
+from os import umask
 from os.path import split
 from datetime import datetime
 
@@ -61,13 +62,19 @@ class HGRepository(repo.VCSRepository):
         for i in xrange(latestrev, actualrev):
             self.info('importing revision %s', i)
             node = changelog.node(i)
+            changeset = unicode(short_hex(node))
+            if execute('Any X WHERE X from_repository R, R eid %(repo)s, X changeset %(cs)s',
+                       {'repo': repoentity.eid, 'cs': changeset}):
+                self.warning('skip revision %s, seems already imported',
+                             changeset)
+                continue
             ctx = repo.changectx(i)
             date = datetime.fromtimestamp(ctx.date()[0])
             #taglist = ctx.tags() #repo.nodetags(node)
             revdata = {'date': date, 'revision': i,
                        'author': unicode(ctx.user(), self.encoding, 'replace'),
                        'description': unicode(ctx.description(), self.encoding, 'replace'),
-                       'changeset': unicode(short_hex(node)),
+                       'changeset': changeset,
                        'branch': unicode(ctx.branch()),
                        }
             self.imported_revision = i # for hooks
@@ -191,6 +198,14 @@ class HGTransaction(object):
     def commit(self):
         if not self.changes:
             raise QueryError('nothing changed')
+        # XXX: make umask configurable
+        oldumask = umask(022)
+        try:
+            self._commit()
+        finally:
+            umask(oldumask)
+
+    def _commit(self):
         # XXX merging branches
         encode = self.repohdlr.encode
         branch = encode(self.revision.get('branch') or u'default')
@@ -207,12 +222,21 @@ class HGTransaction(object):
         # ensure mercurial will use configured repo encoding, not locale's
         # encoding
         # XXX modifying module's global is not very nice but I've no other idea
-        util._encoding = self.repohdlr.encoding
+        try:
+            # mercurial >= 1.3.1
+            from mercurial import encoding
+            encoding.encoding = self.repohdlr.encoding
+        except ImportError:
+            util._encoding = self.repohdlr.encoding
         ctx = context.memctx(self.repo, (p1, p2), msg, self.changes.keys(),
                              self._filectx, author, extra=self.extra)
-        node = self.repo._commitctx(ctx, force=True, force_editor=False,
-                                    empty_ok=True, use_dirstate=False,
-                                    update_dirstate=False)
+        try:
+            # mercurial < 1.3.1
+            node = self.repo._commitctx(ctx, force=True, force_editor=False,
+                                        empty_ok=True, use_dirstate=False,
+                                        update_dirstate=False)
+        except AttributeError:
+            node = self.repo.commitctx(ctx)
         # update revision's record to set correct changeset
         #
         # take care, self.repo != self.session.repo (eg mercurial repo instance
@@ -227,8 +251,13 @@ class HGTransaction(object):
         self.rollback()
 
     def rollback(self):
-        del self._wlock
-        del self._lock
+        try:
+            # mercurial >= 1.3.1
+            self._wlock.release()
+            self._lock.release()
+        except:
+            del self._wlock
+            del self._lock
 
 from logging import getLogger
 from cubicweb import set_log_methods
