@@ -47,9 +47,14 @@ class SVNRepository(repo.VCSRepository):
         #ra_ctx = ra.open2('file://' + self.path, callbacks, None, None)
         latestrev = rev = repoentity.latest_known_revision
         actualrev = fs.youngest_rev(fsrep)
+        execute = repoentity.req.execute
         while rev < actualrev:
             base_root = fs.revision_root(fsrep, rev)
             rev += 1
+            if execute('Any X WHERE X from_repository R, R eid %(repo)s, X revision %(rev)s',
+                       {'repo': repoentity.eid, 'rev': rev}):
+                self.warning('skip revision %s, seems already imported', rev)
+                continue
             self.info('importing content of %s for revision %s', self, rev)
             self.imported_revision = rev # for hooks
             # the base of the comparison
