@@ -6,6 +6,10 @@
 """
 __docformat__ = "restructuredtext en"
 
+from yams.buildobjs import (EntityType, RelationType, SubjectRelation,
+                            String, Int, Bytes)
+
+_ = unicode
 
 class Repository(EntityType):
     """a local subversion repository which will be used as versionned content
@@ -58,6 +62,7 @@ class Revision(EntityType):
 
     from_repository = SubjectRelation('Repository', cardinality='1*', composite='object')
     parent_revision = SubjectRelation('Revision')
+    ancestor_revision = SubjectRelation('Revision')
 
 
 class VersionedFile(EntityType):
