@@ -13,12 +13,12 @@ from copy import copy
 from logilab.mtconverter import html_escape
 
 from cubicweb.selectors import (match_form_params, match_kwargs,
-                                implements, specified_etype_implements)
+                                implements, entity_implements, specified_etype_implements)
 from cubicweb.view import EntityView
 from cubicweb.common import tags
 from cubicweb.web import (Redirect, controller, form,
                           formfields as fields, formwidgets as fwdgs)
-from cubicweb.web.views import autoform, editforms
+from cubicweb.web.views import autoform, editforms, forms
 
 from cubes.vcsfile import IMMUTABLE_ATTRIBUTES
 
@@ -73,8 +73,12 @@ def filter_out_immutable_rels(func):
         for rschema, tschemas, role in func(self, *args, **kwargs):
             if role == 'subject':
                 etype = entity.id
-            else:
+            # take care, according to the wrapped function tschemas is a list or
+            # a single schema
+            elif isinstance(tschemas, list):
                 etype = tschemas[0]
+            else:
+                etype = tschemas
             if '%s.%s' % (etype, rschema) in immutable:
                 continue
             yield rschema, tschemas, role
@@ -90,14 +94,31 @@ class VCRevisionEditionForm(autoform.AutomaticEntityForm):
     inlined_relations = filter_out_immutable_rels(
         autoform.AutomaticEntityForm.inlined_relations)
 
+def available_branches(form):
+    return form.edited_entity.repository.branches()
+
+
+class NewRevisionForm(forms.EntityFieldsForm):
+    id = 'vfnewrevform'
+    __select__ = entity_implements('VersionedFile')
+
+    form_renderer_id = 'base'
+    form_buttons = [fwdgs.SubmitButton()]
+    @property
+    def action(self):
+        return self.build_url('vcsnewrev')
+
+    file = fields.FileField(eidparam=True)
+    branch = fields.StringField(required=True, eidparam=True,
+                                widget=fwdgs.Select, choices=available_branches)
+    msg = fields.StringField(label=_('commit message'), eidparam=True)
+
 
 class VFUploadFormView(form.FormViewMixIn, EntityView):
     """form to upload a new revision of a versioned file"""
     id = 'vfnewrevform'
     __select__ = implements('VersionedFile')
 
-    controller = 'vcsnewrev'
-
     def call(self, **_kwargs):
         # XXX explain why we would want self.rset.rowcount != 1
         for i in xrange(self.rset.rowcount):
@@ -110,24 +131,14 @@ class VFUploadFormView(form.FormViewMixIn, EntityView):
 
     def cell_call(self, row, col, **kwargs):
         entity = self.entity(row, col)
-        branches = entity.repository.branches()
         self.w(u'<div class="iformTitle">')
         self.w(self.form_title(entity))
         self.w(u'</div>')
-        form = self.vreg.select_object('forms', 'base', self.req, entity=entity,
-                                       action=self.build_url(self.controller),
-                                       __redirectpath=entity.rest_path(),
-                                       form_renderer_id='base',
-                                       form_buttons=[fwdgs.SubmitButton()])
-        form.append_field(fields.FileField(name=_('file'), eidparam=True))
-        form.append_field(fields.StringField(name=_('branch'), required=True,
-                                             eidparam=True, widget=fwdgs.Select,
-                                             choices=branches))
-        form.append_field(fields.StringField(label=_('commit message'),
-                                             name='msg', eidparam=True))
+        form = self.vreg.select_object('forms', 'vfnewrevform', self.req, entity=entity,
+                                       __redirectpath=entity.rest_path(),)
         self.additional_form_fields(form, entity)
         # give default values so it won't try to get them from the entity
-        self.w(form.form_render(file=u'', branch=u'', msg=u''))
+        self.w(form.form_render(branch=entity.repository.default_branch(), file=u'', msg=u''))
 
     def additional_form_fields(self, form, entity):
         pass
