@@ -16,8 +16,7 @@ from cubicweb.web.views.idownloadable import IDownloadablePrimaryView
 from cubicweb.web.views.baseviews import MetaDataView
 from cubicweb.web.views.primary import PrimaryView
 
-def render_entity_metadata(self, entity):
-    entity.view('metadata', w=self.w)
+def render_entity_summary(self, entity):
     if entity.description:
         self.w(tags.div(entity.description, klass='summary'))
     if not entity.is_head(entity.rev.branch):
@@ -41,7 +40,7 @@ class DVCPrimaryView(PrimaryView):
                % {'revision': entity.revision,
                   'file': entity.file.view('oneline')})
 
-    render_entity_metadata = render_entity_metadata
+    render_entity_summary = render_entity_summary
 
 
 class VCPrimaryView(IDownloadablePrimaryView):
@@ -52,7 +51,7 @@ class VCPrimaryView(IDownloadablePrimaryView):
                % {'revision': entity.revision,
                   'file': entity.file.view('oneline')})
 
-    render_entity_metadata = render_entity_metadata
+    render_entity_summary = render_entity_summary
 
 
 class VCMetaDataView(MetaDataView):
